@@ -15,6 +15,8 @@ class TaskContext(object):
 
         self.task_state = self.unpack('task_state', TaskState) or TaskState()
         self.pipeline_state = self.unpack('pipeline_state', PipelineState)
+        self.pipeline_state_size = self.pipeline_state.ByteSize() if self.pipeline_state is not None else 0
+
         self._task_meta = {}
         self._task_name = None
 
@@ -92,6 +94,15 @@ class TaskContext(object):
     def get_caller_id(self):
         return None if self._context.caller.identity == "" else self._context.caller.identity
 
+    def set_state(self, obj):
+        self.task_state.internal_state.CopyFrom(pack_any(self._serialiser.to_proto(obj)))
+
+    def get_state(self, default=None):
+        if self.task_state.HasField('internal_state'):
+            return self._serialiser.from_proto(self.task_state.internal_state, default)
+        
+        return default
+
     @staticmethod
     def to_address_and_id(address: Address):
         return f'{address.namespace}/{address.type}', address.id
