@@ -1,6 +1,7 @@
 from statefun_tasks.context import TaskContext
 from statefun_tasks.messages_pb2 import ChildPipeline, TaskRequest, TaskResult, TaskException, Pipeline, TaskActionRequest, TaskAction, TaskStatus
 from statefun_tasks.serialisation import DefaultSerialiser
+from statefun_tasks.storage import StorageBackend
 from statefun_tasks.types import Task, Group, TaskCancelledException
 from statefun_tasks.type_helpers import _create_task_exception
 from statefun_tasks.pipeline_impl.handlers import BeginPipelineHandler, ContinuePipelineHandler, EndPipelineHandler, CancelPipelineHandler
@@ -11,21 +12,22 @@ from typing import Union
 
 
 class _Pipeline(object):
-    def __init__(self, pipeline: list, serialiser=None, is_fruitful=True, events: EventHandlers=None):
+    def __init__(self, pipeline: list, serialiser=None, is_fruitful=True, events: EventHandlers=None, storage: StorageBackend=None):
         self._pipeline = pipeline
         self._serialiser = serialiser if serialiser is not None else DefaultSerialiser()
         self._is_fruitful = is_fruitful
         self._events = events or EventHandlers()
+        self._storage = storage
 
         self._handlers = [
-            BeginPipelineHandler(self._pipeline, self._serialiser),
-            ContinuePipelineHandler(self._pipeline, self._serialiser),
-            CancelPipelineHandler(self._pipeline, self._serialiser),
-            EndPipelineHandler(self._pipeline, self._serialiser)
+            BeginPipelineHandler(self._pipeline, self._serialiser, self._storage),
+            ContinuePipelineHandler(self._pipeline, self._serialiser, self._storage),
+            CancelPipelineHandler(self._pipeline, self._serialiser, self._storage),
+            EndPipelineHandler(self._pipeline, self._serialiser, self._storage)
         ]
 
         self._graph = PipelineGraph(self._pipeline)
-        self._submitter = DeferredTaskSubmitter(self._graph, self._serialiser)
+        self._submitter = DeferredTaskSubmitter(self._graph, self._serialiser, self._storage)
 
     @property
     def events(self) -> EventHandlers:
@@ -43,7 +45,7 @@ class _Pipeline(object):
         return pipeline
 
     @staticmethod
-    def from_proto(pipeline_proto: Pipeline, serialiser, events):
+    def from_proto(pipeline_proto: Pipeline, serialiser, events, storage):
         pipeline = []
 
         for proto in pipeline_proto.entries:
@@ -52,14 +54,14 @@ class _Pipeline(object):
             elif proto.HasField('group_entry'):
                 pipeline.append(Group.from_proto(proto))
 
-        return _Pipeline(pipeline, serialiser=serialiser, events=events)
+        return _Pipeline(pipeline, serialiser=serialiser, events=events, storage=storage)
 
-    def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], task_state: Any=None) -> bool:
+    async def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], task_state: Any=None) -> bool:
         handled = False
 
         for handler in self._handlers:
             if handler.can_handle_message(context, message):
-                should_continue, message = handler.handle_message(context, message, pipeline=self, task_state=task_state)
+                should_continue, message = await handler.handle_message(context, message, pipeline=self, task_state=task_state)
                 handled = True
 
                 if not should_continue:
@@ -88,7 +90,7 @@ class _Pipeline(object):
     def status(self, context: TaskContext):
         return context.pipeline_state.status
 
-    def pause(self, context: TaskContext):
+    async def pause(self, context: TaskContext):
         if context.pipeline_state.status.value not in [TaskStatus.PENDING, TaskStatus.RUNNING, TaskStatus.PAUSED]:
             raise ValueError(f'Pipeline is not in a state that can be paused')
 
@@ -100,21 +102,26 @@ class _Pipeline(object):
             pause_action = TaskActionRequest(id=child_pipeline.id, action=TaskAction.PAUSE_PIPELINE)
             context.pack_and_send(child_pipeline.address, pause_action.id, pause_action)
 
-    def unpause(self, context: TaskContext):
+    async def unpause(self, context: TaskContext):
         if context.pipeline_state.status.value not in [TaskStatus.PENDING, TaskStatus.RUNNING, TaskStatus.PAUSED]:
             raise ValueError(f'Pipeline is not in a state that can be unpaused')
         
         context.pipeline_state.status.value = TaskStatus.Status.RUNNING
         self.events.notify_pipeline_status_changed(context, context.pipeline_state.pipeline, context.pipeline_state.status.value)
 
-        self._submitter.unpause_tasks(context)
+        try:
+            await self._submitter.unpause_tasks(context)
 
-        # tell any child pipelines to resume
-        for child_pipeline in context.pipeline_state.child_pipelines:
-            pause_action = TaskActionRequest(id=child_pipeline.id, action=TaskAction.UNPAUSE_PIPELINE)
-            context.pack_and_send(child_pipeline.address, pause_action.id, pause_action)
+            # tell any child pipelines to resume
+            for child_pipeline in context.pipeline_state.child_pipelines:
+                pause_action = TaskActionRequest(id=child_pipeline.id, action=TaskAction.UNPAUSE_PIPELINE)
+                context.pack_and_send(child_pipeline.address, pause_action.id, pause_action)
+        
+        except Exception as ex:
+            # abort the pipeline if we could not resume the tasks
+            await self.cancel(context, ex)
 
-    def cancel(self, context: TaskContext):
+    async def cancel(self, context: TaskContext, ex=None):
         if context.pipeline_state.status.value not in [TaskStatus.PENDING, TaskStatus.RUNNING, TaskStatus.PAUSED]:
             raise ValueError(f'Pipeline is not in a state that can be cancelled')
 
@@ -128,7 +135,8 @@ class _Pipeline(object):
 
         # construct the cancellation exception to send to caller of this pipeline
         task_request = context.unpack('task_request', TaskRequest) or TaskRequest()
-        cancellation_ex = _create_task_exception(task_request, TaskCancelledException('Pipeline was cancelled'), context.pipeline_state.last_task_state)
+        ex = ex or TaskCancelledException('Pipeline was cancelled')
+        cancellation_ex = _create_task_exception(task_request, ex, context.pipeline_state.last_task_state)
 
         # we move from cancelling to cancelled either by submitting and/or waiting on the finally task...
         finally_task = self._graph.try_get_finally_task(context.get_caller_id())
@@ -137,7 +145,7 @@ class _Pipeline(object):
             if self.get_result_before_finally(context) is None:
 
                 # then we still need to submit the finally task
-                self._submitter.submit_tasks(context, [finally_task], cancellation_ex)
+                await self._submitter.submit_tasks(context, [finally_task], cancellation_ex)
 
             # set result before finally to our task cancellation exception
             self.save_result_before_finally(context, cancellation_ex)
