@@ -1,6 +1,6 @@
 from statefun_tasks.context import TaskContext
 from statefun_tasks.pipeline_impl.handlers import PipelineMessageHandler
-from statefun_tasks.types import Task, Group
+from statefun_tasks.types import Task, Group, TasksException
 from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, TaskStatus
 from typing import Union
 
@@ -14,7 +14,7 @@ class ContinuePipelineHandler(PipelineMessageHandler):
             and context.pipeline_state.status.value in [TaskStatus.RUNNING, TaskStatus.PAUSED] \
                 and isinstance(message, (TaskResult, TaskException))
 
-    def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], pipeline, **kwargs):
+    async def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], pipeline: '_Pipeline', **kwargs):
         task_result_or_exception = message
         caller_id = context.get_caller_id()
 
@@ -25,26 +25,26 @@ class ContinuePipelineHandler(PipelineMessageHandler):
         pipeline.events.notify_pipeline_task_finished(context, task_result_or_exception)
 
         # release deferred tasks if they can now run
-        self.submitter.release_tasks(context, caller_id, task_result_or_exception)
+        await self.submitter.release_tasks(context, caller_id, task_result_or_exception)
 
         # get the next step of the pipeline to run (if any)
         current_step, next_step, group, empty_group = self.graph.get_next_step_in_pipeline(caller_id)
 
         # if this task is part group then we need to record the results so we can aggregate later
         if group is not None:
-            self.result_aggregator.add_result(context, caller_id, task_result_or_exception)
+            await self.result_aggregator.add_result(context, group, caller_id, task_result_or_exception)
 
             # once the group is complete aggregate the results
             if group.is_complete():
-                task_result_or_exception = self.result_aggregator.aggregate(context, group)
+                task_result_or_exception = await self.result_aggregator.aggregate(context, group)
 
                 # pause the pipeline if this completed group is a wait
                 if group.is_wait:
-                    pipeline.pause(context)
+                    await pipeline.pause(context)
         else:
             # pause the pipeline if this task is a wait
             if current_step.is_wait:  
-                pipeline.pause(context)
+                await pipeline.pause(context)
 
         # if we got an exception then the next step is the finally_task if there is one (or none otherwise)
         if isinstance(task_result_or_exception, TaskException):
@@ -72,7 +72,7 @@ class ContinuePipelineHandler(PipelineMessageHandler):
 
         if any(tasks):
             # split into tasks to call now and those to defer if max parallelism is exceeded
-            self.submitter.submit_tasks(context, tasks, task_result_or_exception=task_result_or_exception, max_parallelism=max_parallelism)
+            await self.submitter.submit_tasks(context, tasks, task_result_or_exception=task_result_or_exception, max_parallelism=max_parallelism)
 
         else:
             last_step = self._pipeline[-1]
@@ -90,3 +90,4 @@ class ContinuePipelineHandler(PipelineMessageHandler):
 
         # continue
         return True, task_result_or_exception
+    
\ No newline at end of file
